@@ -35,19 +35,18 @@ VSERVER_NAME = 'test'
 CREATE_URL = '{}/slb/virtual-server/{}/port/'.format(BASE_URL, VSERVER_NAME)
 OBJECT_URL = '{}/slb/virtual-server/{}/port/80+http'.format(BASE_URL, VSERVER_NAME)
 ALL_URL = '{}/slb/virtual-server/{}/port/'.format(BASE_URL, VSERVER_NAME)
+OK_RESP = {'response': {'status': 'OK'}}
 
 
 class TestVirtualPort(unittest.TestCase):
 
     def setUp(self):
         self.client = client.Client(HOSTNAME, '30', 'fake_username', 'fake_password')
-        self.maxDiff = None
 
     @responses.activate
     def test_virtual_port_create_no_params(self):
         responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
-        json_response = {'response': {'status': 'OK'}}
-        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
+        responses.add(responses.POST, CREATE_URL, json=OK_RESP, status=200)
         params = {
             'port':
             {
@@ -64,7 +63,7 @@ class TestVirtualPort(unittest.TestCase):
             service_group_name='pool1'
         )
 
-        self.assertEqual(resp, json_response)
+        self.assertEqual(resp, OK_RESP)
         self.assertEqual(len(responses.calls), 2)
         self.assertEqual(responses.calls[1].request.method, responses.POST)
         self.assertEqual(responses.calls[1].request.url, CREATE_URL)
@@ -73,8 +72,7 @@ class TestVirtualPort(unittest.TestCase):
     @responses.activate
     def test_virtual_port_create_with_params(self):
         responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
-        json_response = {'response': {'status': 'OK'}}
-        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
+        responses.add(responses.POST, CREATE_URL, json=OK_RESP, status=200)
         params = {
             'port':
             {
@@ -117,7 +115,7 @@ class TestVirtualPort(unittest.TestCase):
             use_rcv_hop=True,
         )
 
-        self.assertEqual(resp, json_response)
+        self.assertEqual(resp, OK_RESP)
         self.assertEqual(len(responses.calls), 2)
         self.assertEqual(responses.calls[1].request.method, responses.POST)
         self.assertEqual(responses.calls[1].request.url, CREATE_URL)
@@ -144,8 +142,8 @@ class TestVirtualPort(unittest.TestCase):
     @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
     @responses.activate
     def test_virtual_port_update_no_params(self, mocked_get):
-        mocked_get.return_value = {"foo": "bar"}
         responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
+        mocked_get.return_value = {"foo": "bar"}
         json_response = {"foo": "bar"}
         responses.add(responses.POST, OBJECT_URL, json=json_response, status=200)
         params = {
@@ -157,7 +155,7 @@ class TestVirtualPort(unittest.TestCase):
                 "port-number": 80,
                 "template-persist-source-ip": None,
                 "template-persist-cookie": None,
-                "extended-stats": 1,
+                "extended-stats": 1
             }
         }
 
@@ -176,8 +174,7 @@ class TestVirtualPort(unittest.TestCase):
     @responses.activate
     def test_virtual_port_create_with_templates(self, mocked_get):
         responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
-        json_response = {'response': {'status': 'OK'}}
-        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
+        responses.add(responses.POST, CREATE_URL, json=OK_RESP, status=200)
         protocol = self.client.slb.virtual_server.vport.HTTP
         if protocol.lower() == 'http':
             params = {
@@ -196,8 +193,7 @@ class TestVirtualPort(unittest.TestCase):
                     'template-persist-source-ip': 'test_s_pers_template',
                     'udp_template': 'test_udp_template',
                     'template-virtual-port': 'template_vp',
-                    'template-http': None,
-                    'template-policy': 'template_pl',
+                    'template-policy': 'template_pl'
                 }
             }
         else:
@@ -218,7 +214,7 @@ class TestVirtualPort(unittest.TestCase):
                     'udp_template': 'test_udp_template',
                     'template-virtual-port': 'template_vp',
                     'template-tcp': 'template_tcp',
-                    'template-policy': 'template_pl',
+                    'template-policy': 'template_pl'
                 }
             }
 
@@ -243,7 +239,7 @@ class TestVirtualPort(unittest.TestCase):
             },
         )
 
-        self.assertEqual(resp, json_response)
+        self.assertEqual(resp, OK_RESP)
         self.assertEqual(len(responses.calls), 2)
         self.assertEqual(responses.calls[1].request.method, responses.POST)
         self.assertEqual(responses.calls[1].request.url, CREATE_URL)
@@ -253,8 +249,7 @@ class TestVirtualPort(unittest.TestCase):
     @responses.activate
     def test_virtual_port_create_with_partial_templates(self, mocked_get):
         responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
-        json_response = {'response': {'status': 'OK'}}
-        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)
+        responses.add(responses.POST, CREATE_URL, json=OK_RESP, status=200)
         protocol = self.client.slb.virtual_server.vport.HTTP
         if protocol.lower() == 'http':
             params = {
@@ -272,9 +267,7 @@ class TestVirtualPort(unittest.TestCase):
                     'template-persist-cookie': 'test_c_pers_template',
                     'template-persist-source-ip': 'test_s_pers_template',
                     'udp_template': 'test_udp_template',
-                    'template-virtual-port': 'template_vp',
-                    'template-http': None,
-                    'template-policy': None,
+                    'template-virtual-port': 'template_vp'
                 }
             }
         else:
@@ -318,7 +311,7 @@ class TestVirtualPort(unittest.TestCase):
             },
         )
 
-        self.assertEqual(resp, json_response)
+        self.assertEqual(resp, OK_RESP)
         self.assertEqual(len(responses.calls), 2)
         self.assertEqual(responses.calls[1].request.method, responses.POST)
         self.assertEqual(responses.calls[1].request.url, CREATE_URL)
@@ -327,8 +320,8 @@ class TestVirtualPort(unittest.TestCase):
     @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
     @responses.activate
     def test_virtual_port_update_with_params(self, mocked_get):
-        mocked_get.return_value = {"foo": "bar"}
         responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
+        mocked_get.return_value = {"foo": "bar"}
         json_response = {"foo": "bar"}
         responses.add(responses.POST, OBJECT_URL, json=json_response, status=200)
         params = {
@@ -381,8 +374,8 @@ class TestVirtualPort(unittest.TestCase):
     @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
     @responses.activate
     def test_virtual_port_update_with_templates(self, mocked_get):
-        mocked_get.return_value = {"foo": "bar"}
         responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
+        mocked_get.return_value = {"foo": "bar"}
         json_response = {"foo": "bar"}
         responses.add(responses.POST, OBJECT_URL, json=json_response, status=200)
         protocol = self.client.slb.virtual_server.vport.HTTP
@@ -406,9 +399,7 @@ class TestVirtualPort(unittest.TestCase):
                     'template-persist-cookie': 'test_c_pers_template',
                     'template-persist-source-ip': 'test_s_pers_template',
                     'udp_template': 'test_udp_template',
-                    'template-virtual-port': 'template_vp',
-                    'template-http': None,
-                    'template-policy': None,
+                    'template-virtual-port': 'template_vp'
                 }
             }
         else:
@@ -431,9 +422,7 @@ class TestVirtualPort(unittest.TestCase):
                     'template-persist-cookie': 'test_c_pers_template',
                     'template-persist-source-ip': 'test_s_pers_template',
                     'udp_template': 'test_udp_template',
-                    'template-virtual-port': 'template_vp',
-                    'template-tcp': None,
-                    'template-policy': None,
+                    'template-virtual-port': 'template_vp'
                 }
             }
         resp = self.client.slb.virtual_server.vport.update(
@@ -464,19 +453,152 @@ class TestVirtualPort(unittest.TestCase):
         self.assertEqual(responses.calls[1].request.url, OBJECT_URL)
         self.assertEqual(json.loads(responses.calls[1].request.body), params)
 
+    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
+    @responses.activate
+    def test_virtual_port_replace_with_params(self, mocked_get):
+        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
+        mocked_get.return_value = {"foo": "bar"}
+        json_response = {"foo": "bar"}
+        responses.add(responses.PUT, OBJECT_URL, json=json_response, status=200)
+        params = {
+            'port':
+            {
+                'auto': 1,
+                'extended-stats': 1,
+                'name': 'test1_VPORT',
+                'ipinip': 1,
+                'no-dest-nat': 1,
+                'pool': 'test_nat_pool',
+                'port-number': 80,
+                'protocol': 'http',
+                'service-group': 'pool1',
+                'ha-conn-mirror': 1,
+                'conn-limit': 50000,
+                'tcp_template': 'test_tcp_template',
+                'template-persist-cookie': 'test_c_pers_template',
+                'template-persist-source-ip': 'test_s_pers_template',
+                'udp_template': 'test_udp_template',
+                'use-rcv-hop-for-resp': 1,
+            }
+        }
+
+        resp = self.client.slb.virtual_server.vport.replace(
+            virtual_server_name=VSERVER_NAME,
+            name='test1_VPORT',
+            protocol=self.client.slb.virtual_server.vport.HTTP,
+            port='80',
+            service_group_name='pool1',
+            s_pers_name="test_s_pers_template",
+            c_pers_name="test_c_pers_template",
+            status=1,
+            autosnat=True,
+            ipinip=True,
+            ha_conn_mirror=1,
+            no_dest_nat=1,
+            conn_limit=50000,
+            source_nat_pool="test_nat_pool",
+            tcp_template="test_tcp_template",
+            udp_template="test_udp_template",
+            use_rcv_hop=True,
+        )
+        self.assertEqual(resp, json_response)
+        self.assertEqual(len(responses.calls), 2)
+        self.assertEqual(responses.calls[1].request.method, responses.PUT)
+        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)
+        self.assertEqual(json.loads(responses.calls[1].request.body), params)
+
+    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
+    @responses.activate
+    def test_virtual_port_replace_with_templates(self, mocked_get):
+        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
+        mocked_get.return_value = {"foo": "bar"}
+        json_response = {"foo": "bar"}
+        responses.add(responses.PUT, OBJECT_URL, json=json_response, status=200)
+        protocol = self.client.slb.virtual_server.vport.HTTP
+        if protocol.lower() == 'http':
+            params = {
+                'port':
+                {
+                    'auto': 1,
+                    'extended-stats': 1,
+                    'name': 'test1_VPORT',
+                    'ipinip': 1,
+                    'no-dest-nat': 0,
+                    'use-rcv-hop-for-resp': 0,
+                    'pool': 'test_nat_pool',
+                    'port-number': 80,
+                    'protocol': 'http',
+                    'service-group': 'pool1',
+                    'ha-conn-mirror': 1,
+                    'conn-limit': 50000,
+                    'tcp_template': 'test_tcp_template',
+                    'template-persist-cookie': 'test_c_pers_template',
+                    'template-persist-source-ip': 'test_s_pers_template',
+                    'udp_template': 'test_udp_template',
+                    'template-virtual-port': 'template_vp'
+                }
+            }
+        else:
+            params = {
+                'port':
+                {
+                    'auto': 1,
+                    'extended-stats': 1,
+                    'name': 'test1_VPORT',
+                    'ipinip': 1,
+                    'no-dest-nat': 0,
+                    'use-rcv-hop-for-resp': 0,
+                    'pool': 'test_nat_pool',
+                    'port-number': 80,
+                    'protocol': 'http',
+                    'service-group': 'pool1',
+                    'ha-conn-mirror': 1,
+                    'conn-limit': 50000,
+                    'tcp_template': 'test_tcp_template',
+                    'template-persist-cookie': 'test_c_pers_template',
+                    'template-persist-source-ip': 'test_s_pers_template',
+                    'udp_template': 'test_udp_template',
+                    'template-virtual-port': 'template_vp'
+                }
+            }
+        resp = self.client.slb.virtual_server.vport.replace(
+            virtual_server_name=VSERVER_NAME,
+            name='test1_VPORT',
+            protocol=self.client.slb.virtual_server.vport.HTTP,
+            port='80',
+            service_group_name='pool1',
+            s_pers_name="test_s_pers_template",
+            c_pers_name="test_c_pers_template",
+            status=1,
+            autosnat=True,
+            ipinip=True,
+            ha_conn_mirror=1,
+            no_dest_nat=0,
+            conn_limit=50000,
+            source_nat_pool="test_nat_pool",
+            tcp_template="test_tcp_template",
+            udp_template="test_udp_template",
+            virtual_port_templates={
+                'template-virtual-port': 'template_vp'
+            },
+            use_rcv_hop=False,
+        )
+        self.assertEqual(resp, json_response)
+        self.assertEqual(len(responses.calls), 2)
+        self.assertEqual(responses.calls[1].request.method, responses.PUT)
+        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)
+        self.assertEqual(json.loads(responses.calls[1].request.body), params)
+
     @responses.activate
     def test_virtual_port_delete(self):
         responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
-        json_response = {
-            'response': {'status': 'OK'}
-        }
-        responses.add(responses.DELETE, OBJECT_URL, json=json_response, status=200)
+        responses.add(responses.DELETE, OBJECT_URL, json=OK_RESP, status=200)
 
         resp = self.client.slb.virtual_server.vport.delete(
             VSERVER_NAME, 'test1_VPORT', self.client.slb.virtual_server.vport.HTTP, '80'
         )
 
-        self.assertEqual(resp, json_response)
+        self.assertEqual(resp, OK_RESP)
         self.assertEqual(len(responses.calls), 2)
         self.assertEqual(responses.calls[1].request.method, responses.DELETE)
         self.assertEqual(responses.calls[1].request.url, OBJECT_URL)
